package client.model;

import client.gui.UpdateGui;
import client.model.exception.ConnectException;
import client.model.exception.ParseException;
import client.model.executor.Executor;
import message.GetLastMessages;
import message.GetOnlineMembers;
import message.Message;

import java.io.*;
import java.net.InetSocketAddress;
import java.net.Socket;

/**
 * User: sumboid
 * Date: 5/16/11
 * Time: 8:24 PM
 */
public final class Client
{
    private static final String configFileName = "client.messages";

    private Socket socket = null;
    private String name = null;

    private final static Executor executor = new Executor();

    private ObjectOutputStream outputStream;
    private ObjectInputStream inputStream;

    private UpdateGui gui;

    public void connect(final String ip, final int port) throws ConnectException
    {
        socket = new Socket();

        try
        {
            executor.initExecutor(new FileReader(configFileName));
        }
        catch (FileNotFoundException e)
        {
            System.err.println(configFileName + " not found!");
            return;
        }
        catch (ParseException e)
        {
            System.err.println(e.getMessage());
            return;
        }

        try
        {
            socket.connect(new InetSocketAddress(ip, port));
        }
        catch (IOException e)
        {
            throw new ConnectException("Can't connect to server");
        }
    }

    public void makeStreams() throws ConnectException
    {
        try
        {
            outputStream = new ObjectOutputStream(socket.getOutputStream());
        }
        catch (IOException e)
        {
            throw new ConnectException("Can't make output stream");
        }

        try
        {
            inputStream = new ObjectInputStream(socket.getInputStream());
        }
        catch (IOException e)
        {
            throw new ConnectException("Can't make input stream");
        }
    }

    public void sendName(final String nick) throws ConnectException
    {
        makeStreams();

        name = nick;

        try
        {
            outputStream.writeObject(name);
        }
        catch (IOException e)
        {
            throw new ConnectException("Can't send nickname");
        }

        try
        {
            Object answer = inputStream.readObject();
            executor.processMessage(answer);
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        catch (ClassNotFoundException e)
        {
            throw new ConnectException("You have old version of Message library");
        }
    }

    public void getMessages()
    {
        new Thread(new Runnable()
        {
            public void run()
            {
                listen();
            }
        }).start();
    }

    private void listen()
    {
        while(true)
        {
            try
            {
                Object object = inputStream.readObject();
                executor.processMessage(object);
            }
            catch (IOException e)
            {
                gui.printMessage("Disconected!");
                e.printStackTrace();
                return;
            }
            catch (ClassNotFoundException e)
            {
                gui.printMessage("You don't have Message library that Server use!");
                return;
            }
            catch (ConnectException e)
            {
                System.err.println(e.getMessage());
            }
        }

    }

    public void sendMessage(final String message)
    {
        if(null == message)
        {
            throw new IllegalArgumentException("message == null");
        }

        try
        {
            outputStream.writeObject(new Message(name, message));
        }
        catch (IOException e)
        {
            gui.printMessage("Disconected!");
        }
    }

    public void setGui(final UpdateGui _gui)
    {
        gui = _gui;
        executor.setGui(_gui);
    }

    public void getMembersList()
    {
        try
        {
            outputStream.writeObject(new GetOnlineMembers());
        }
        catch (IOException e)
        {
            gui.printMessage("Disconected!");
        }
    }

    public void getLastMessages()
    {
        try
        {
            outputStream.writeObject(new GetLastMessages());
        }
        catch (IOException e)
        {
            gui.printMessage("Disconected!");
        }
    }
}

/*
public final class Client
{
    private Socket socket;
    private String name;

    private ObjectOutputStream outputStream;
    private ObjectInputStream inputStream;

    private UpdateGui client.gui;

    public void connect(final String ip, final int port) throws ConnectException {
        socket = new Socket();
        try
        {
            socket.connect(new InetSocketAddress(ip, port));
        }
        catch (IOException e)
        {
            throw new ConnectException("Can't connect to server");
        }
    }

    public void makeStreams() throws ConnectException
    {
        try
        {
            outputStream = new ObjectOutputStream(socket.getOutputStream());
        }
        catch (IOException e)
        {
            throw new ConnectException("Can't make output stream");
        }

        try
        {
            inputStream = new ObjectInputStream(socket.getInputStream());
        }
        catch (IOException e)
        {
            throw new ConnectException("Can't make input stream");
        }
    }

    public void sendName(final String nick) throws ConnectException
    {
        makeStreams();
        name = nick;
        try
        {
            outputStream.writeObject(name);
        }
        catch (IOException e)
        {
            throw new ConnectException("Can't send nickname");
        }

        try
        {
            Object answer = inputStream.readObject();
            if(answer instanceof BadMessage)
            {
                throw new ConnectException("This nickname is busy");
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        catch (ClassNotFoundException e)
        {
            throw new ConnectException("You have old version of Message library");
        }
    }

    public void getMessages()
    {
        new Thread(new Runnable()
        {
            public void run()
            {
                listen();
            }
        }).start();
    }

    private void listen()
    {
        while(true)
        {
            try
            {
                Object object = inputStream.readObject();

                if(object instanceof CoreMessage)
                {
                    if(object instanceof MemberIsOffline)
                    {
                        removeMemberFromList(((MemberIsOffline) object).getName());
                    }
                    else if(object instanceof MemberIsOnline)
                    {
                        addMemberFromList(((MemberIsOnline) object).getName());
                    }

                    client.gui.printMessage(object.toString());

                }
                else if(object instanceof MembersList)
                {
                    client.gui.globalSetMembersList(((MembersList) object).getMembers());
                }
            }
            catch (IOException e)
            {
                client.gui.printMessage("Disconected!");
                e.printStackTrace();
                return;
            }
            catch (ClassNotFoundException e)
            {
                client.gui.printMessage("You don't have Message library that Server use!");
                return;
            }
        }

    }

    public void printMessage(final String message)
    {
        try
        {
            outputStream.writeObject(new Message(name, message));
        }
        catch (IOException e)
        {
            client.gui.printMessage("Disconected!");
        }
    }

    public void setGui(final UpdateGui _gui)
    {
        client.gui = _gui;
    }

    public void getMembersList()
    {
        try
        {
            outputStream.writeObject(new GetOnlineMembers());
        }
        catch (IOException e)
        {
            client.gui.printMessage("Disconected!");
        }
    }

    private void removeMemberFromList(final String name)
    {
        client.gui.userIsOffline(name);
    }

    private void addMemberFromList(final String name)
    {
        client.gui.userIsOnline(name);
    }

    private void sendCoreMembersList(final HashSet<String> members)
    {
        client.gui.globalSetMembersList(members);
    }
}

*/