package server;

import message.*;
import org.apache.log4j.Logger;

import java.io.*;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.util.ArrayList;
import java.util.concurrent.*;

/**
 * User: sumboid
 * Date: 5/16/11
 * Time: 7:25 PM
 */

final class Server implements Runnable
{
    private ServerSocket serverSocket;
    private ConcurrentHashMap<String, ClientSocket> online = new ConcurrentHashMap<String, ClientSocket>();
    private BlockingQueue<CoreMessage> lastMessages = new ArrayBlockingQueue<CoreMessage>(10);

    private static final Logger logger = Logger.getLogger(Server.class);

    public Server(final String ip, final int port) throws IOException
    {
        logger.info("Make socket");
        serverSocket = new ServerSocket();
        logger.info("Bind");
        serverSocket.bind(new InetSocketAddress(ip, port));
    }

    public void run()
    {
        final ClientSocket clientSocket;
        try
        {
            logger.info("Waiting connection");
            clientSocket = new ClientSocket(serverSocket);
            logger.info("Connect with: " + clientSocket.getAddress());
        }
        catch (final IOException e)
        {
            logger.info("!! Catch IOException. Stack trace:");
            logger.info(e.getStackTrace());
            e.printStackTrace();
            return;
        }

        String name;

        try
        {
            name = clientSocket.getName();
        }
        catch (IOException e)
        {
            logger.info("!! Catch IOException. Stack trace:");
            logger.info(e.getStackTrace());
            e.printStackTrace();
            return;
        }
        catch (ClassNotFoundException e)
        {
            logger.info("!! Catch ClassNotFoundException. Stack trace:");
            logger.info(e.getStackTrace());
            e.printStackTrace();
            return;
        }

        logger.info("User connected with nick: " + name);

        if(online.containsKey(name))
        {
            try
            {
                logger.info("This nick already used: " + name);
                clientSocket.sendMessage(new BadMessage("This nick already used"));
            }
            catch (IOException e)
            {
                logger.info("!! Catch IOException. Stack trace:");
                logger.info(e.getStackTrace());
                e.printStackTrace();
            }

            return;
        }

        try
        {
            logger.info("This nick already used: " + name);
            clientSocket.sendMessage(new GoodMessage());
        }
        catch (IOException e)
        {
            logger.info("!! Catch IOException. Stack trace:");
            logger.info(e.getStackTrace());
            e.printStackTrace();
            return;
        }

        logger.info("Send message that new user is online: " + name);
        sendMessage(new MemberIsOnline(name));
        online.put(name, clientSocket);

        final String finalName = name;

        new Thread(new Runnable()
        {
            public void run()
            {
                getMessages(finalName, clientSocket);
            }
        }).start();
    }

   /*
    * Work with client.model
    * Get messages and send it to all
    */

    private void removeUser(final String name)
    {
        logger.info("remove user: " + name);
        try
        {
            online.get(name).close();
        }
        catch (IOException e)
        {
            logger.info("!! Catch IOException. Stack trace:");
            logger.info(e.getStackTrace());
            e.printStackTrace();
        }
        online.remove(name);
        sendMessage(new MemberIsOffline(name));
    }

    private void getMessages(final String name, final ClientSocket clientSocket)
    {
        while (true)
        {
            Object message = null;

            try
            {
                message = clientSocket.getMessage();
            }
            catch(EOFException e) //this is normal exception
            {
                removeUser(name);
                return;
            }
            catch (IOException e) //this is normal exception
            {
                removeUser(name);
                return;
            }
            catch (ClassNotFoundException e)
            {
                logger.info("!! Catch ClassNotFoundException. Stack trace:");
                logger.info(e.getStackTrace());
                e.printStackTrace();
            }

            try
            {
                if(message instanceof Message)
                {
                    logger.info("Send \'" + message.toString() + "\' to all");
                    sendMessage((Message) message);
                }
                else if(message instanceof GetOnlineMembers)
                {
                    logger.info("Send members list to " + clientSocket.getName());
                    clientSocket.sendMessage(new MembersList(online.keySet()));
                }
                else if(message instanceof GetLastMessages)
                {
                    clientSocket.sendMessage(new LastMessages(lastMessages.toArray(new CoreMessage[lastMessages.size()])));
                }
                else
                {
                    clientSocket.sendMessage(new BadMessage("Server can't parse your message. Sorry"));
                }
            }
            catch (final ClassCastException e)
            {
                logger.info("!! Catch ClassCastException. Stack trace:");
                logger.info(e.getStackTrace().toString());
                e.printStackTrace();
            }
            catch (IOException e)
            {
                removeUser(name);
                return;
            }
            catch (ClassNotFoundException e)
            {
                logger.info("!! Catch ClassNotFoundException. Stack trace:");
                logger.info(e.getStackTrace());
                e.printStackTrace();
            }
        }
    }

    private void sendMessage(final CoreMessage message)
    {
        while(true)
        {
            try
            {
                lastMessages.add(message);
                break;
            }
            catch(final IllegalStateException e)
            {
                lastMessages.poll();
            }
        }

        final ArrayList<ClientSocket> stream = new ArrayList<ClientSocket>(online.values());
        ArrayBlockingQueue<Runnable> queue = new ArrayBlockingQueue<Runnable>(100);
        ThreadPoolExecutor pool = new ThreadPoolExecutor(1, 100, 0, TimeUnit.SECONDS, queue, new RejectedExecutionHandler()
        {
            public void rejectedExecution(Runnable r, ThreadPoolExecutor executor)
            {
                if (!executor.isShutdown())
                {
                    try
                    {
                        executor.getQueue().put(r);
                    }
                    catch (InterruptedException ex)
                    {
                        ex.printStackTrace();
                    }
                }
            }
        });

        for(int i = 0; i < stream.size(); i++)
        {
            final int index = i;
            pool.execute(new Runnable()
            {
                public void run()
                {
                    try
                    {
                        if(null != stream.get(index))
                        {
                            stream.get(index).sendMessage(message);
                        }
                    }
                    catch (IOException e)
                    {
                        //block is empty, because this situation getMessage()-thread handle
                    }
                }
            });
        }
    }
}
